# Copyright 2018 miruka
# This file is part of lunasync, licensed under LGPLv3.

import csv
import shutil
from pathlib import Path
from typing import Optional

from appdirs import user_config_dir
from pkg_resources import resource_filename

from . import __about__

DEFAULT_FILE = resource_filename(__about__.__name__, "data/default_config.csv")
FILE         = f"%s/%s.csv" % (user_config_dir(__about__.__project_name__),
                               __about__.__pkg_name__)
FIELDS = ["tag_search", "filter", "labels", "booru"]
SUBS   = []

def reload(path: Optional[str] = None) -> None:
    path = Path(path or FILE).expanduser()

    if not path.is_file():
        path.parent.mkdir(parents=True, exist_ok=True)
        shutil.copyfile(DEFAULT_FILE, path)

    with open(path, "r") as file:
        SUBS.extend(list(
            csv.DictReader(
                (row for row in file if row and not row.startswith("#")),
                fieldnames = FIELDS
            )
        ))



# Copyright 2018 miruka
# This file is part of lunasync, licensed under LGPLv3.

# pylint: disable=wrong-import-position

from copy import copy

import lunafind

LOG = copy(lunafind.LOG)

from .__about__ import __doc__
from . import config, savedata
config.reload()

from .main import sync



# Copyright 2018 miruka
# This file is part of lunasync, licensed under LGPLv3.

from typing import Dict, Optional, Sequence

from lunafind import Stream

from . import config, savedata


def sync(subs: Optional[Sequence[Dict[str, Optional[str]]]] = None,
         only_for_labels: Sequence[str]  = (),
         force_full:      bool           = False,
         overwrite:       bool           = False,
         warn:            bool           = True) -> int:

    downloaded = 0

    for sub in subs or config.SUBS:
        sub = {k: "" if v is None or v.strip() == "%" else v.strip()
               for k, v in sub.items()}

        if only_for_labels and \
           not any(l in only_for_labels for l in sub["labels"].split()):
            continue

        savedata.reload()

        try:
            sub_data = savedata.DATA[str(sub)]
        except KeyError:
            savedata.DATA[str(sub)] = {"last_id": 0, "success": None}
            sub_data                = savedata.DATA[str(sub)]

        stream = Stream(sub["tag_search"],
                        pages  = "all",
                        limit  = 200,
                        prefer = sub["booru"] or None).filter(sub["filter"])

        if not force_full:
            stream = stream.stop_if("id:<=%s" % sub_data["last_id"])

        sub_data["success"] = False

        try:
            newest_id   = sub_data["last_id"]
            newest_post = next(stream)
            newest_id   = newest_post.id

            newest_post.write(overwrite=overwrite, warn=warn)
            stream.write(overwrite=overwrite, warn=warn)

        except StopIteration:
            pass

        except Exception:
            savedata.write()
            raise

        try:
            # Verify everything has been processed (no CTRL-C happened):
            next(stream)
        except StopIteration:
            sub_data["success"] = True
            sub_data["last_id"] = newest_id

        savedata.write()

        downloaded += stream.downloaded
        print()

    return downloaded



# Copyright 2018 miruka
# This file is part of lunasync, licensed under LGPLv3.

import json
from pathlib import Path
from typing import Any, Dict, Optional

from appdirs import user_data_dir
from atomicfile import AtomicFile

from lunafind.utils import jsonify

from . import __about__

FILE = "%s/%s.csv" % (user_data_dir(__about__.__project_name__),
                      __about__.__pkg_name__)

DATA: Dict[str, Dict[str, Any]] = {}


def reload(path: Optional[str] = None) -> None:
    path = Path(path or FILE).expanduser()

    if not path.exists():
        path.parent.mkdir(parents=True, exist_ok=True)
        path.write_text("{}")

    DATA.update(json.loads(path.read_text()))


def write(path: Optional[str] = None) -> None:
    path = Path(path or FILE).expanduser()

    with AtomicFile(path, "w") as file:
        file.write(jsonify(DATA, indent=4))



# Copyright 2018 miruka
# This file is part of lunasync, licensed under LGPLv3.

r"""Usage: lunasync [LABEL]... [options]

Keep downloaded booru searches up-to-date, similar to Danbooru subscriptions.

Subscriptions are defined in the config file (see `--print-config-path`),
and will be downloaded/updated using `lunafind`.
The number of posts downloaded will be output on stdout.

Arguments:
  LABEL
    If label arguments are passed, only synchronize subscriptions that have
    at least one of the indicated label.
    This can be useful for e.g. scheduling certain subscription syncs
    with cron at different intervals.
    With no argument, synchronize everything.

Options:
  -f, --force
    Force a complete re-check for subscriptions.
    Without this, the sync will stop upon reaching the most recently
    downloaded post of previous syncs (if there were any).

  -q, --quiet-skip
    Do not warn when skipping download of already existing files.

  -o, --overwrite
    Force download and overwrite any files that already exist.


  --config PATH
    Use `PATH` as configuration file instead of the default location.

  --print-config-path
    Show the configuration file path.
    If the file doesn't exist, a default one is automatically copied.

  -h, --help
    Show this help.

  -V, --version
    Show the program version."""

import sys
import time
from typing import List, Optional

import docopt

import lunafind

from . import __about__, config, sync


def main(argv: Optional[List[str]] = None) -> None:
    argv = argv if argv is not None else sys.argv[1:]

    try:
        args = docopt.docopt(
            __doc__, help=False, argv=argv, version=__about__.__version__
        )
    except docopt.DocoptExit:
        if len(sys.argv) > 1:
            print(lunafind.TERM.red("Invalid command syntax, check help:\n"))

        lunafind.utils.print_colored_help(__doc__, exit_code=10)

    if args["--config"]:
        config.FILE = args["--config"]
        config.reload()

    if args["--help"]:
        lunafind.utils.print_colored_help(__doc__)

    if args["--print-config-path"]:
        print(config.FILE)
        sys.exit()

    downloaded = sync(
        only_for_labels = args["LABEL"],
        force_full      = args["--force"],
        overwrite       = args["--overwrite"],
        warn            = not args["--quiet-skip"]
    )

    time.sleep(0.2)  # Other threads might manage to print after this else
    print(downloaded)
